{ lib, ... }:
let
  waw = import ./networks/waw.hackerspace.pl.nix;
  networks = import ./networks.nix { inherit lib; };
in {
  services.kea = {
    dhcp4 = {
      enable = true;
      settings = {
        interfaces-config = { interfaces = [ "lan" "bms" ]; };

        lease-database = {
          name = "/var/lib/kea/dhcp4.leases";
          persist = true;
          type = "memfile";
        };

        rebind-timer = 300;
        renew-timer = 150;
        valid-lifetime = 600;

        # yanked from https://kea.readthedocs.io/en/latest/arm/dhcp4-srv.html#setting-fixed-fields-in-classification
        # if i understand correctly, the logic is reversed to what isc-dhcp
        # config did, but result should be the same
        client-classes = [{
          name = "ipxe_efi_x64";
          test = "option[93].hex == 0x0009";
          boot-file-name = "netboot.xyz.efi";
        }];

        subnet4 = [
          { # general members area lan
            id = 1;
            subnet = "10.8.0.0/16";
            pools = [{ pool = "10.8.0.20 - 10.8.0.199"; }];
            reservations-out-of-pool = false;
            reservations-in-subnet = true;
            authoritative = true;

            next-server = "10.8.1.2";

            option-data = [
              {
                name = "routers";
                data = "10.8.1.2";
              }
              {
                name = "domain-name-servers";
                data = "10.8.1.2";
              }
              {
                name = "boot-file-name";
                data = "netboot.xyz.kpxe";
              }
            ];

            reservations = networks.keaReservationsPreformat waw.hosts;
          }
          { # bms
            id = 2;
            subnet = "10.11.1.0/24";
            pools = [{ pool = "10.11.1.100 - 10.11.1.200"; }];
            reservations-out-of-pool = false;
            reservations-in-subnet = true;
            authoritative = true;

            option-data = [
              {
                name = "routers";
                data = "10.11.1.1";
              }
              {
                name = "domain-name-servers";
                data = "10.11.1.1";
              }
            ];
          }
        ];
      };
    };
  };

  users.users.kea = {
    group = "kea";
    isSystemUser = true;
  };
  users.groups.kea = {};

  systemd.services.kea-dhcp4-server.serviceConfig = {
    UMask = lib.mkForce "0033";
    DynamicUser = lib.mkForce false;
  };
}
