{ lib }:

let
  keaReservationsPreformat = lib.attrsets.mapAttrsToList (name: val: {
    hostname = val.hostname;
    hw-address = name;
    ip-address = val.ip;
  });

  dnsSoa = zoneMeta: with zoneMeta; ''
  ${name} IN SOA ${ns} ${admin} (
    ${toString serial}
    ${toString refresh}
    ${toString retry}
    ${toString expire}
    ${toString minimum}
  )'';

  hostRecord = (_: val:
    "${val.hostname} A ${val.ip}"
    + (lib.optionalString (val ? aliases)
        "${(lib.strings.concatMapStrings (a: "\n${a} CNAME ${val.hostname}") val.aliases)}"
    )
  );

  genZone = zone: ''
  $ORIGIN .
  $TTL 180
  ${dnsSoa zone.dns.meta}
      NS ${zone.dns.meta.ns}

  $ORIGIN ${zone.dns.meta.name}.
  $TTL ${toString zone.dns.meta.ttl}

  '' +
  (lib.optionalString (zone.dns ? extraRecords) zone.dns.extraRecords) +
  (lib.strings.concatLines (lib.attrsets.mapAttrsToList hostRecord zone.hosts));
in {
    inherit keaReservationsPreformat genZone;
}
