{ config, lib, pkgs, ... }:

let
  postgresPkg = pkgs.postgresql_16;
  numCPUs = 32; # not increasing as we have other tenants/services
  # modify as needed when performing upgrades
  pgUpgradeScript =
    let
      from = "14";
      to = "16";
      pg_from = "postgresql_" + from;
      pg_to = "postgresql_" + to;
      pg_pkg_from = pkgs.${pg_from};
      pg_pkg_to = pkgs.${pg_to};
    in
    pkgs.writeScriptBin "pg-upgrade-${from}-${to}" ''
      #!${pkgs.runtimeShell}
      export PGDATAOLD=/var/lib/postgresql/${from}
      export PGDATANEW=/var/lib/postgresql/${to}
      export PGBINOLD=${pg_pkg_from}/bin
      export PGBINNEW=${pg_pkg_to}/bin
      ${pg_pkg_to}/bin/pg_upgrade
    '';
in
{
  services.postgresql = {
    enable = true;
    package = postgresPkg;
    enableTCPIP = true;
    initdbArgs = [ "--encoding='UTF8'" "--lc-collate='C'" "--lc-ctype='C'" ];
    settings = {
      max_connections = 300;
      shared_buffers = "16GB";
      temp_buffers = "128MB";
      work_mem = "128MB";
      maintenance_work_mem = "258MB";
      effective_io_concurrency = 32; # ssd, guess
      maintenance_io_concurrency = 100; # ssd, guess
      max_worker_processes = numCPUs;
      max_parallel_workers = numCPUs;
      max_parallel_maintenance_workers = 4;
      wal_level = "logical";
      wal_sync_method = "fsync"; # slightly faster (per pg_test_fsync) AND safer
      full_page_writes = "off"; # partial writes impossible on zfs
      wal_init_zero = "off"; # useless on CoW
      wal_recycle = "off";  # same
      random_page_cost = 2.0; # ssd, TODO maybe even lower?
    };

    authentication = pkgs.lib.mkOverride 10 ''
      local   all               all                                 trust
      host    all               all               127.0.0.1/32      trust
      host    all               all               ::1/128           trust
      host    synapse,mediarepo synapse,mediarepo 185.236.240.0/24  scram-sha-256
    '';

    # comment out these two sections when performing upgrades.
    # pg_upgrade doesn't like it when the new db version has any contents of its own
    ensureDatabases = [ "synapse" "mediarepo" ];
    ensureUsers = [
      {
        name = "synapse";
        ensureDBOwnership = true;
      }
      {
        name = "mediarepo";
        ensureDBOwnership = true;
      }
    ];
  };

  fileSystems."/var/lib/postgresql" =
    { device = "snowflake/postgres";
      fsType = "zfs";
    };

  environment.systemPackages = [
    postgresPkg
  #  pgUpgradeScript
  ];
  networking.firewall.allowedTCPPorts = [ config.services.postgresql.settings.port ];
  services.zfs.trim.enable = true;
}
